/*  batchprocessingwizard_v12.js: Batch processing script for BrainVoyager QX 2.3 and higher

    This batch processing bundle consists of batchprocessingwizard.js, batchprocessingwizard.ui and batchprocessingwizard_v10.dylib, place all in 
    /Documents/BVQXExtensions/Plugins_32 or 64/ folder (for Mac OS X: also Plugins_64 is possible).
    Functionality: Create *.fmr/*.dmr and/or preprocess multiple *.fmr files.
    Usage: In the BrainVoyager menu, select 'Plugins' > 'batchprocessingwizard v1.1'.
    Note: This version is for BrainVoyager QX 3.2 or higher, since PreprocessdFileName now includes the path.
    Latest update: 
    21-12-2011 (v1.2): corrected small bug for creating VTC files in readTextFileIntoProjectNamesArray()
    22-11-2011 (v1.2): added VTC/VDW/MTC creation and VTC preprocessing
    08-02-2011 (v1.1): changed path for creation of preprocessed files, added more printing of the preprocessing parameters to the log tab.
    12-10-2010: added intra-session alignment    

        ~ help functions
        getNativeProcessingFiles
        writeToTextFile
        readTextFileIntoProjectNamesArray
        getPath
        getNormalisedProcessingFiles
        getSurfaceProcessingFiles
        getVTCVDWVMRName
        getMTCVMRName
        getFileName
        noErrors
        getProjectTypeString

        ~ `core' functions
        startProcessing
        startSliceScanTimeCorrection
        startMotionCorrection
        startTemporalFiltering
        createProjects
        createVTCsOrVDWs
        startPreprocessingVTCs
        createMTCs

    Hester Breman, Brain Innovation, 2007-2011.
*/

var scriptObj = new Object; 
scriptObj.scriptNameForUser = "batchprocessingwizard"; 
scriptObj.dialogFileName = "batchprocessingwizard_v12.ui"; 
scriptObj.dialogAccessName = "batchDlg"; 
BrainVoyagerQX.PrintToLog("Batch processing wizard started...");

var projfilenames;// = new Array();
var projfilenamessize = 0;
var targetfilenames = new Array();
var targetfilenamessize = 0;
var cancel = false;
const FORBVQX22 = true;
const PROJECTTYPE_UNKNOWN       = 0;
const PROJECTTYPE_FMR           = 1;
const PROJECTTYPE_FMR_MOSAIC    = 2;
const PROJECTTYPE_FMR_LOOPING   = 3;
const PROJECTTYPE_DMR           = 4;
const PROJECTTYPE_DMR_MOSAIC    = 5;
const UNKNOWN_SPACE             = 0;
const NATIVE_SPACE              = 1;
const NORMALISED_SPACE          = 2;
const SURFACE_SPACE             = 3;
const AC_PC_SPACE               = 2;
const TALAIRACH_SPACE           = 3;
const INTERPOLATION_UNKNOWN     = 0;
const INTERPOLATION_NN          = 1;
const INTERPOLATION_TRILINEAR   = 2;
const INTERPOLATION_SINC        = 3;
const TEXTFILE_FOR_PROJECTCREATION  = 1;
const TEXTFILE_FOR_NORMALISATION    = 2;
const TEXTFILE_FOR_VTCPREPROCESSING = 3;
const TEXTFILE_FOR_SURFACEPROCESSING = 4;


// global widgets
var startButton, cancelButton;
// widgets on global tab
var cmbSelectProcSpace, createProjectsBox, doMotionCorrBox, doSliceTimeCorrBox, doSpatialFilterBox, doTempFilterBox;
var mocoOnlyBox, mocoAndISABox;
var thpAllBox, thpLinearBox, thpNoneBox, tlpNoBox, tlpYesBox, sfNoBox, sfYesBox;
var chbCreateVTCVDWs, chbVTCSpatialSmoothing, chbVTCTemporalFiltering;
var rbtnVTCNoTHP, rbtnVTCLinearHP, rbtnVTCAllTHP, rbtnVTCNoTLP, rbtnVTCTLP;
var chbCreateMTCs;
// widgets on create FMR/DMR projects tab
var createAMRbox, filetypeBox, bigendianBox, selectSrcFilesBox, nrSlicesBox, nrVolsBox, skipNrVolsBox, xMosResBox, yMosResBox, isMosaicBox, nrMosaicVols, imgXResBox, imgYResBox, isDMRbox, isFMRbox, stcPrefixBox, sliceloopsBox, isFloatBox, isIntBox, bytesPerPixelBox; 
var istCheckBox, istValueBox, trCheckBox, trValueBox, xCheckBox, xValueBox, yCheckBox, yValueBox, zCheckBox, zValueBox;
// widgets on FMR preprocessing parameters tab
var mocoLessDataBox, mocoLogBox, mocoInterpBox, mocoNrIterBox, mocoTargetVolBox, mocoISABox;
var scsInterpBox, scsAscBox, scsAscIntBox, scsAscInt2Box, scsDescBox, scsDescIntbox, scsDescInt2Box;
var sgsMMbox, sgsPxBox, sgsSizeBox, thpCycBox, thpHzBox, thpSizeBox, lpSecBox, lpTRBox, lpSizeBox;
// widgets on create VTC/VDW tab
var rbtnSelectVTC, rbtnSelectVDW, rbtnVTCVDWdtypeFloat, rbtnVTCVDWdtypeInt, rbtnSelectNativeSpace, rbtnSelectACPCSpace, rbtnSelectTalSpace;
var chbExtendCerebellum, rbtnResolution1mm3, rbtnResolution2mm3, rbtnResolution3mm3, cmbVTCVDWInterpolation, spbIntensityThreshold;
// widgets on preprocess VTC tab
var rbtnVTCmm, rbtnVTCvox, dspbFWHMSpatialSmooth, rbtnVTCHz, rbtnVTCCyc; 
var dspbVTCFWHMTHP, rbtnVTCSec, rbtnVTCDataPoints, dspbVTCFWHMTGS;
// widgets on create MTC tab
var dspbMTCFrom, dspbMTCTo;
// widgets on files tab
var nroffilesBox, getFileButton, selectFilesNowBox, selectTextFileBox;
var selectFilesNowBox2, selectTextFileBox2, btnGetVTCVDWnames, btnVTCVDWGetVMRName, lblVTCVDWVMRname, spbNrVTCVDWs;
var selectFilesNowBox3, selectTextFileBox3, btnGetMTCnames, btnMTCGetVMRName, lblMTCVMRname, spbNrMTCs; 


// ~~~~~~~~~~~~~~~~~~~~~~~ File I/O ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.getNativeProcessingFiles = function() {

	BrainVoyagerQX.PrintToLog("Get files..."); 
    var filetype = "*.*";
    if (createProjectsBox.checked == 1) {
        var rawfiletype = filetypeBox.currentText;
        if (rawfiletype == "DICOM") {
            filetype = "*.dcm";
        } else if (rawfiletype == "SIEMENS") {
            filetype = "*.IMA";
        } else if (rawfiletype == "PHILIPS_REC") {
            filetype = "*.REC";    
        } else if (rawfiletype == "GE_I") {
            filetype = "*.*";
        } else if (rawfiletype == "GE_MR") {
            filetype = "*.MR";
        } else if (rawfiletype == "ANALYZE") { 
            filetype = "*.hdr"; 
        } 
    } else {
        filetype = "*.fmr";
    }
    
    // select files manually
    projfilenames = new Array();
    if (selectFilesNowBox.checked == 1) {
    
        var nroffiles = nroffilesBox.value;
        if (nroffiles < 1) BrainVoyagerQX.PrintToLog("Please note: number of selected files is 0.");
        BrainVoyagerQX.PrintToLog("Start selecting: " + nroffiles + " files..."); 
        var filecounter;
        // loop over indicated number of files
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            // get raw data filename or *.fmr filename
            
            var projfilename = this.getFileName(filetype);
            if (projfilename.length > 0) projfilenames.push(projfilename);     
            
            if (mocoISABox.checked) {
            
                BrainVoyagerQX.SetPluginStringParameter("Command", "GetTargetFileName");
                BrainVoyagerQX.SetPluginStringParameter("FileType", filetype);
                BrainVoyagerQX.ExecutePlugin();
                if (this.noErrors()) {
                    var targetfilename = BrainVoyagerQX.GetPluginStringParameter("TargetFilename");
                    BrainVoyagerQX.PrintToLog("Selected target filename: " + targetfilename);
                    targetfilenamessize = targetfilenames.push(targetfilename);
                } else {
                    success = false;
                }
            }
        }
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "input");
        BrainVoyagerQX.PrintToLog("Files selected. If all parameters have been set, the Start button can be clicked.");
        
    // read filenames from textfile    
    } else if (selectTextFileBox.checked == 1) {
    
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetTextFileName");
        BrainVoyagerQX.PrintToLog("Start selecting text file with source filenames..."); 
        BrainVoyagerQX.ExecutePlugin();
        if (this.noErrors()) {
            var textfilename = BrainVoyagerQX.GetPluginStringParameter("Filename");
            BrainVoyagerQX.PrintToLog("Selected text filename: " + textfilename);
            this.readTextFileIntoProjectNamesArray(textfilename, TEXTFILE_FOR_PROJECTCREATION);
        } else {
            success = false;
        }

    }    
}

scriptObj.writeToTextFile = function(projfilenames, kindoffiles, stage, targetfilenames) { 

    var filecounter;
    var success;
    
    // write the textfile to the path of the first file
    var filetype, name;
    if ((kindoffiles != TEXTFILE_FOR_NORMALISATION) && (kindoffiles != TEXTFILE_FOR_SURFACEPROCESSING)) {
        filetype = projfilenames[0].substring((projfilenames[0].length-3), projfilenames[0].length); 
        name = String(this.getPath(projfilenames[0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
    } else {
        filetype = projfilenames[0][0].substring((projfilenames[0][0].length-3), projfilenames[0][0].length);
        name = String(this.getPath(projfilenames[0][0]) + "filenames_" + stage + "_" + filetype.toUpperCase() + ".txt");
    } 
    BrainVoyagerQX.PrintToLog("Start writing file: " + name);
    var filenamesfile = new QFile(name);     
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.WriteOnly)); 
    var textstr = new QTextStream(filenamesfile); 
    textstr.writeDouble(projfilenames.length);
    textstr.writeString("\n");
    if (kindoffiles == TEXTFILE_FOR_PROJECTCREATION) {
        BrainVoyagerQX.PrintToLog("Write native space names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter] + "\n");
        }
        if ((stage == "isa") && (targetfilenames != null)) {
            textstr.writeString("TARGETFMRs" + "\n");
            for (filecounter = 0; filecounter < targetfilenames.length; filecounter++) {
                textstr.writeString(targetfilenames[filecounter] + "\n");
            }
        }    
    } else if (kindoffiles == TEXTFILE_FOR_NORMALISATION) {
        BrainVoyagerQX.PrintToLog("Write space code...");
        switch (stage) {
            case "create_native_vtc": textstr.writeString("space: native" + "\n"); break;
            case "create_acpc_vtc": textstr.writeString("space: ac-pc" + "\n"); break;
            case "create_tal_vtc": textstr.writeString("space: talairach" + "\n"); break;
            default: textstr.writeString("0" + "\n"); break;
        }
        BrainVoyagerQX.PrintToLog("Write normalised space names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter][0] + "\n"); // source file (FMR or DMR)
            textstr.writeString(projfilenames[filecounter][1] + "\n"); // initial alignment file
            textstr.writeString(projfilenames[filecounter][2] + "\n"); // fine alignment file
            if ((stage == "create_acpc_vtc")|| (stage == "create_tal_vtc")) {
                textstr.writeString(projfilenames[filecounter][3] + "\n"); // AC-PC alignment file
            }
            if (stage == "create_tal_vtc") {
                 textstr.writeString(projfilenames[filecounter][4] + "\n"); // Talairach transformation file
            }
        }
    } else if (kindoffiles == TEXTFILE_FOR_VTCPREPROCESSING) {
        BrainVoyagerQX.PrintToLog("Write normalised space names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter] + "\n");
        }    
    } else if (kindoffiles == TEXTFILE_FOR_SURFACEPROCESSING) {
        BrainVoyagerQX.PrintToLog("Write surface space names...");
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            textstr.writeString(projfilenames[filecounter][0] + "\n"); // source file (VTC or MTC)
            textstr.writeString(projfilenames[filecounter][1] + "\n"); // surface file
        }
    } else {
        BrainVoyagerQX.PrintToLog("Kind of files to write to textfile unknown.");
        success = false;
    }
    filenamesfile.close();
    BrainVoyagerQX.PrintToLog("Wrote " + name + ".");
    return success;
}

scriptObj.readTextFileIntoProjectNamesArray = function(textfilename, kindoffiles) {

    var filecounter;    
    var success = true;
    BrainVoyagerQX.PrintToLog("Start reading: " + textfilename);
    var name = String(textfilename); // circumvention of problem with implicit strings
    var filenamesfile = new QFile(name);     
    filenamesfile.open(new QIODevice.OpenMode(QIODevice.ReadOnly));
    var textstr = new QTextStream(filenamesfile); 
    var nroffiles = parseInt(textstr.readLine());
    BrainVoyagerQX.PrintToLog("Number of projects: " + nroffiles);
    BrainVoyagerQX.PrintToLog("Kind of projects: " + ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) ? "project creation/preprocessing" :
                                                    (kindoffiles == TEXTFILE_FOR_VTCPREPROCESSING) ? "vtc preprocessing related" :
                                                    (kindoffiles == TEXTFILE_FOR_NORMALISATION) ? "normalisation" :
                                                    (kindoffiles == TEXTFILE_FOR_SURFACEPROCESSING) ? "surface related" : 
                                                    "unknown")); 

    if ((kindoffiles == TEXTFILE_FOR_PROJECTCREATION) || (kindoffiles == TEXTFILE_FOR_VTCPREPROCESSING)) { 
        projfilenames = new Array();
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            var filename = textstr.readLine();
            BrainVoyagerQX.PrintToLog("File " + (filecounter+1) + ":\n" + filename);
            projfilenamessize = projfilenames.push(filename);
        } // end for
	} // end if
    if (kindoffiles == TEXTFILE_FOR_PROJECTCREATION) {    
        if (textstr.readLine() == "TARGETFMRs") {
            for (filecounter = 0; filecounter < nroffiles; filecounter++) {
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog("Targetfile " + (filecounter+1) + ":\n" + filename);
                targetfilenamessize = targetfilenames.push(filename);        
            }
        }
    } else if (kindoffiles == TEXTFILE_FOR_NORMALISATION) {
    
        var space;
        projfilenames = new Array(nroffiles);
        var groupnames;
        var spaceLine = textstr.readLine();     
        if (spaceLine.indexOf("native") > -1) {
            space = NATIVE_SPACE;
        } else if (spaceLine.indexOf("ac-pc") > -1) {    
            space = AC_PC_SPACE;
        } else if (spaceLine.indexOf("talairach") > -1) {
            space = TALAIRACH_SPACE;
        } else {
            space = UNKNOWN_SPACE;
            success = false;
            BrainVoyagerQX.PrintToLog("Target space for VTC/VDW not recognised.");
        }
        BrainVoyagerQX.PrintToLog("VTC/VDW target space: " + space);
        if (success) {
            for (filecounter = 0; filecounter < nroffiles; filecounter++) {
                
                var groupnames;
                if (space == NATIVE_SPACE) {
                    groupnames = new Array(3); 
                } else if (space == AC_PC_SPACE) {    
                    groupnames = new Array(4); 
                } else if (space == TALAIRACH_SPACE) {
                    groupnames = new Array(5); 
                }
                BrainVoyagerQX.PrintToLog("Files for project " + (filecounter+1) + ":");
                // *.fmr/*.dmr
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog(filename); 
                groupnames[0] = filename;
                // *_IA.trf
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog(filename); 
                groupnames[1] = filename;
                // *_FA.trf
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog(filename); 
                groupnames[2] = filename;    
                if ((space == AC_PC_SPACE) || (space == TALAIRACH_SPACE)) { 
                    // *ACPC.trf
                    var filename = textstr.readLine();
                    BrainVoyagerQX.PrintToLog(filename); 
                    groupnames[3] = filename;
                    // *.tal          
                    if (space == TALAIRACH_SPACE) {
                        var filename = textstr.readLine();
                        BrainVoyagerQX.PrintToLog(filename);
                        groupnames[4] = filename;
                    } 
                }
                projfilenames[filecounter] = groupnames;
                
            } // end for
        } // end if success
    } else if (kindoffiles == TEXTFILE_FOR_SURFACEPROCESSING) {

        projfilenames = new Array(nroffiles);
        if (success) {
            for (filecounter = 0; filecounter < nroffiles; filecounter++) {
                
                BrainVoyagerQX.PrintToLog("Files for surface " + (filecounter+1) + ":");
                var groupnames = new Array(2);
                // *.vtc
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog(filename); 
                groupnames[0] = filename;
                // *.srf
                var filename = textstr.readLine();
                BrainVoyagerQX.PrintToLog(filename); 
                groupnames[1] = filename;
                // save names in global array
                projfilenames[filecounter] = groupnames;                
            } 
        } 
    
    } else {
        BrainVoyagerQX.PrintToLog("Kind of files unknown.");
    }
    filenamesfile.close();
    return success;
}

scriptObj.getPath = function(filename) { 

	var start = 0; // start searching from begin 
	var last = filename.lastIndexOf("/");
	var path = filename.substring(start, (last+1)); // now return path without filename
	return path;
}


scriptObj.getNormalisedProcessingFiles = function() {

    var success = true;
	var isvdw = false;
    var projfiletype = "*.*";
    if (chbCreateVTCVDWs.checked && rbtnSelectVDW.checked) {
        projfiletype = "*.dmr"; // vdw creation
        isvdw = true;
    } else if (chbCreateVTCVDWs.checked) {
        projfiletype = "*.fmr"; // at least vtc creation
    } else {
        projfiletype = "*.vtc"; // only vtc preprocessing
    }
    
    // select files manually
    BrainVoyagerQX.PrintToLog("Get files..."); 

    if (selectFilesNowBox2.checked == 1) {
    
        var nroffiles = spbNrVTCVDWs.value;
        if (nroffiles < 1) BrainVoyagerQX.PrintToLog("Please note: number of selected files is 0.");
        
        if (!chbCreateVTCVDWs.checked) {
        
        // only VTC preprocessing
            var stage = "preprocess_vtc";
            projfilenames = new Array(nroffiles);  
            BrainVoyagerQX.PrintToLog("Start selecting " + nroffiles + " VTC file(s)..."); 
            var filecounter, srcfile;
            for (filecounter = 0; filecounter < nroffiles; filecounter++) {
                var filename = this.getFileName("*.vtc");
                if (filename.length > 0) projfilenames[filecounter] = filename;  
            }
            if (FORBVQX22 && (projfilenames.length > 0)) {
                BrainVoyagerQX.PrintToLog("Write selected names to text file, stage: " + stage);
                this.writeToTextFile(projfilenames, TEXTFILE_FOR_VTCPREPROCESSING, stage);
            }
        
        } else {
            var stage = "create_native_vtc";
            projfilenames = new Array(nroffiles);  
            var nrofprojfiles = (rbtnSelectACPCSpace.checked ? 4 : (rbtnSelectTalSpace.checked ? 5 : 3));     
            var projectfiles;
            BrainVoyagerQX.PrintToLog("Start selecting " + nroffiles + " project(s)..."); 
            var filecounter, srcfile, iafile, fafile, acpcfile, talfile;
            for (filecounter = 0; filecounter < nroffiles; filecounter++) {
            
                projectfiles = new Array();
                // get *.fmr/*.dmr/*.vtc filename, initial and fine alignment
                var filename = this.getFileName(projfiletype);
                if (filename.length > 0) projectfiles.push(filename);  
                var filename = this.getFileName("*_IA.trf");
                if (filename.length > 0) projectfiles.push(filename);  
                var filename = this.getFileName("*_FA.trf");
                if (filename.length > 0) projectfiles.push(filename);              
                // optional: AC-PC alignment
                if ((rbtnSelectACPCSpace.checked) || (rbtnSelectTalSpace.checked)) {
                    var filename = this.getFileName("*ACPC.trf");
                    if (filename.length > 0) projectfiles.push(filename);
                    // optional: Talairach
                    if (rbtnSelectTalSpace.checked) {
                        var filename = this.getFileName("*.tal");
                        if (filename.length > 0) projectfiles.push(filename);
                        stage = "create_tal_vtc";
                    } else {
                        stage = "create_acpc_vtc";
                    }
                } 
                if (projectfiles.length == nrofprojfiles) {
                    projfilenames[filecounter] = projectfiles;
                    BrainVoyagerQX.PrintToLog("Storing project files for subject " + (filecounter+1).toString() + ":\n " + projectfiles.toString());
                } else {
                    success = false;
                    BrainVoyagerQX.PrintToLog("Number of normalisation files (" + projectfiles.length + ") not equal to required number of files (" + nrofprojfiles + "):\n " + projectfiles.toString());
                }
            }
            if (FORBVQX22) {
                BrainVoyagerQX.PrintToLog("Write selected names to text file, stage: " + stage);
                this.writeToTextFile(projfilenames, TEXTFILE_FOR_NORMALISATION, stage);
            }
        }
        BrainVoyagerQX.PrintToLog("Files selected. If all parameters have been set, the Start button can be clicked."); 
        
    // read filenames from textfile    
    } else if (selectTextFileBox2.checked == 1) {
    
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetTextFileName");
        BrainVoyagerQX.PrintToLog("Start selecting text file with source filenames..."); 
        BrainVoyagerQX.ExecutePlugin();
        if (this.noErrors()) {
            var textfilename = BrainVoyagerQX.GetPluginStringParameter("Filename");
            BrainVoyagerQX.PrintToLog("Selected text filename: " + textfilename);
            success = this.readTextFileIntoProjectNamesArray(textfilename, TEXTFILE_FOR_NORMALISATION);
        } else {
            success = false;
        }
    }    
    return success; // does not return projfilenames because that is a global variable (as long as it lasts)
}

scriptObj.getSurfaceProcessingFiles = function() {
    
    var filename;
    var success = true;
    
    // select files manually
    BrainVoyagerQX.PrintToLog("Get files..."); 

    if (selectFilesNowBox3.checked == 1) {
    
        var nroffiles = spbNrMTCs.value;
        if (nroffiles < 1) BrainVoyagerQX.PrintToLog("Please note: number of selected files is 0.");
        projfilenames = new Array(nroffiles);  
        var filecounter, vtcfile;
        for (filecounter = 0; filecounter < nroffiles; filecounter++) {
        
            var groupnames = new Array(2);
            // get *.vtc and *.srf filenames
            filename = this.getFileName("*.vtc");
            
            if (filename.length > 0) groupnames[0] = filename;            
            filename = this.getFileName("*.srf");
            if (filename.length > 0) groupnames[1] = filename;        
            projfilenames[filecounter] = groupnames;
        }
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_SURFACEPROCESSING, "vtc_for_mtc"); 
        BrainVoyagerQX.PrintToLog("Files selected. If all parameters have been set, the Start button can be clicked.");

    // read filenames from textfile    
    } else if (selectTextFileBox3.checked == 1) {
    
        BrainVoyagerQX.SetPluginStringParameter("Command", "GetTextFileName");
        BrainVoyagerQX.PrintToLog("Start selecting text file with source filenames..."); 
        BrainVoyagerQX.ExecutePlugin();
        if (this.noErrors()) {
            var textfilename = BrainVoyagerQX.GetPluginStringParameter("Filename");
            BrainVoyagerQX.PrintToLog("Selected text filename: " + textfilename);
            success = this.readTextFileIntoProjectNamesArray(textfilename, TEXTFILE_FOR_SURFACEPROCESSING);
        } else {
            success = false;
        }
    }    
    return success; // does not return projfilenames because that is a global variable (as long as it lasts)

}

scriptObj.getVTCVDWVMRName = function() {

    var vmrname = this.getFileName("*.vmr");
    lblVTCVDWVMRname.text = vmrname;
}
 
  
scriptObj.getMTCVMRName = function() {

    var vmrname = this.getFileName("*.vmr");
    lblMTCVMRname.text = vmrname; 
}


scriptObj.getFileName = function(filetype) {

    var filename = "";
    BrainVoyagerQX.SetPluginStringParameter("Command", "GetFileName");
    BrainVoyagerQX.SetPluginStringParameter("FileType", filetype); 
    BrainVoyagerQX.ExecutePlugin();
    if (this.noErrors()) {
        filename = BrainVoyagerQX.GetPluginStringParameter("Filename");
        BrainVoyagerQX.PrintToLog("Selected: " + filename);
    }
    return filename;
}
    
// ~~~~~~~~~~~~~~~~~~~~~~~ help functions ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 

scriptObj.noErrors = function() {

    var noerror;
    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr != -1) {
        noerror = true;
    } else {
        var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
        BrainVoyagerQX.PrintToLog(errormsg);
        noerror = false;        
    }
    return noerror;
}     

scriptObj.getProjectTypeString = function(projecttype) {

    switch (projecttype) {
    
        case PROJECTTYPE_FMR:           return "FMR";
        case PROJECTTYPE_FMR_MOSAIC:    return "FMR mosaic";
        case PROJECTTYPE_FMR_LOOPING:   return "FMR slices x time";
        case PROJECTTYPE_DMR:           return "DMR";
        case PROJECTTYPE_DMR_MOSAIC:    return "DMR mosaic";
        default:                        return "Onbekend";

    }
}
    
    

// ~~~~~~~~~~~~~~~~~~~~~~~ main processing function ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startProcessing = function() {

    var success;
    BrainVoyagerQX.PrintToLog("Start processing...");
    if (projfilenames.length == 0) {
        BrainVoyagerQX.PrintToLog("No input files have been selected. Process cancelled.");
    } else {
    
        if (cmbSelectProcSpace.currentIndex == NATIVE_SPACE) {
    
            if (createProjectsBox.checked == 1) {
                this.createProjects();
            } 

            if (doSliceTimeCorrBox.checked == 1) {
                this.startSliceScanTimeCorrection();
            } 
                
            if (doMotionCorrBox.checked == 1) {
                this.startMotionCorrection();
            }
            
            if (doTempFilterBox.checked == 1) {
                this.startTemporalFiltering();
            }
            
            if (doSpatialFilterBox.checked == 1) {
                this.startSpatialFiltering();
            }
        } else if (cmbSelectProcSpace.currentIndex == NORMALISED_SPACE) {
        
            if (chbCreateVTCVDWs.checked) {   
                success = this.createVTCsOrVDWs();
            }
            
            if (chbVTCSpatialSmoothing.checked || chbVTCTemporalFiltering.checked) {
                this.startPreprocessingVTCs();
            }
 
        } else if (cmbSelectProcSpace.currentIndex == SURFACE_SPACE) {
        
            if (chbCreateMTCs.checked) {
                success = this.createMTCs();
            }
        
        } else {
            BrainVoyagerQX.PrintToLog("No processing space selected. Process cancelled.");
        }
    }
    this.batchDlg.close();
} 

 
// ~~~~~~~~~~~~~~~~~~~~~~~ FMR Preprocessing ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
scriptObj.startSliceScanTimeCorrection = function() {

	BrainVoyagerQX.PrintToLog("Start slice scan time correction...");
    var interpolationmethod = scsInterpBox.currentIndex;  
    if (interpolationmethod == 3) { // 3 = "Select interpolation method"
        cancel = true;
        BrainVoyagerQX.PrintToLog("No interpolation method selected for slice scan time correction.");
    } else {
        BrainVoyagerQX.PrintToLog("Interpolation method parameter: " + interpolationmethod); 
    }

    var sliceorder;
    if (scsAscBox.checked == 1) {
        sliceorder = 0;
    } else if (scsAscIntBox.checked == 1) {
        sliceorder = 1;
    } else if (scsAscInt2Box.checked == 1) {
        sliceorder = 2;
    } else if (scsDescBox.checked == 1) {
        sliceorder = 10;
    } else if (scsDescIntbox.checked == 1) {
        sliceorder = 11;
    } else if (scsDescInt2Box.checked == 1) {
        sliceorder = 12;
    }
    BrainVoyagerQX.PrintToLog("Slice order parameter: " + sliceorder);
    
    if (!cancel) {
        var filecounter;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyagerQX.PrintToLog("Open " + projfilenames[filecounter]);
            var bvqxproject = BrainVoyagerQX.OpenDocument(projfilenames[filecounter]);
            bvqxproject.CorrectSliceTiming(sliceorder, interpolationmethod);
//            var resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
            var resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
            BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
            projfilenames[filecounter] = resultfilename; // place new name back in old name array
            bvqxproject.Close();
        }
        BrainVoyagerQX.PrintToLog("Finished slice scan time correction.");
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "slicescantimecorrection");
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}
}

scriptObj.startMotionCorrection = function() { 
    
    var success = true;
    // check parameters
    var targetVol = mocoTargetVolBox.value;
    var interpolationOpt = mocoInterpBox.currentIndex;
    if (interpolationOpt < 0) cancel = true;
    var reducedData = mocoLessDataBox.checked;
    var nrIterations = mocoNrIterBox.value;
    var movies = true;
    var writeLog = mocoLogBox.checked;    
    var doISA = mocoISABox.checked;
    BrainVoyagerQX.PrintToLog("Selected options for motion correction: ");
    BrainVoyagerQX.PrintToLog("Target volume: " + targetVol);
    BrainVoyagerQX.PrintToLog("Interpolation method: " + mocoInterpBox.currentIndex + ", which should be " + mocoInterpBox.currentText);   
    BrainVoyagerQX.PrintToLog("Reduced data: " + reducedData);
    BrainVoyagerQX.PrintToLog("Number of iterations: " + nrIterations);
    BrainVoyagerQX.PrintToLog("Movies: yes");
    BrainVoyagerQX.PrintToLog("Write extended log file: " + writeLog);
    BrainVoyagerQX.PrintToLog("Perform intra-session alignment: " + doISA);

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        if (doISA) {
            if (projfilenames.length == targetfilenames.length){
            // perform motion correction with intra-session alignment
                 for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                    BrainVoyagerQX.PrintToLog("Open " + projfilenames[filecounter]);
                    bvqxproject = BrainVoyagerQX.OpenDocument(projfilenames[filecounter]);
                    success = bvqxproject.CorrectMotionTargetVolumeInOtherRunEx(targetfilenames[filecounter], targetVol, interpolationOpt, (!reducedData), nrIterations, movies, writeLog);
                    if (success) {
                        resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
//                        resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
                        BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
                        projfilenames[filecounter] = resultfilename; // place new name back in old name array
                    } else {
                        BrainVoyagerQX.PrintToLog("Motion correction and intra-session alignment of this project did not succeed.");
                    }
                    bvqxproject.Close();
                }
            }
        } else {
            for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
                BrainVoyagerQX.PrintToLog("Open " + projfilenames[filecounter]);
                bvqxproject = BrainVoyagerQX.OpenDocument(projfilenames[filecounter]);
                success = bvqxproject.CorrectMotionEx(targetVol, interpolationOpt, (!reducedData), nrIterations, movies, writeLog);
                if (success) {
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    //resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyagerQX.PrintToLog("Motion correction of this project did not succeed.");
                }
                bvqxproject.Close();
            }
        }
        BrainVoyagerQX.PrintToLog((doISA ? "Finished motion correction and intra-session alignment" : "Finished motion correction."));
        if (FORBVQX22) {
            this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, (doISA ? "isa" : "motioncorrection"), (doISA ? targetfilenames : null));
        }
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled: no valid interpolation option.");
	}
}

scriptObj.startTemporalFiltering = function() { 

    var success = true;
    var thpUnit, thpValue, lpUnit, lpValue;
    var allTHP = thpAllBox.checked; 
    var linearTHP = thpLinearBox.checked; 
    var noTHP = thpNoneBox.checked;
    var noTLP = tlpNoBox.checked; 
    var doTLP = tlpYesBox.checked;
    if (thpCycBox.checked) {
        thpUnit = "cycles";
    } else {
        thpUnit = "Hz";
    }
    var thpValue = thpSizeBox.value;
    if (lpSecBox.checked) {
        lpUnit = "s";
    } else {
        lpUnit = "TR";
    }
    lpValue = lpSizeBox.value;
    if (allTHP) {
        BrainVoyagerQX.PrintToLog("Selected options for temporal high-pass filtering: ");
        BrainVoyagerQX.PrintToLog("Unit: " + thpUnit);
        BrainVoyagerQX.PrintToLog("Value: " + thpValue);
    }
    if (doTLP) {
        BrainVoyagerQX.PrintToLog("Selected options for temporal low-pass filtering: ");
        BrainVoyagerQX.PrintToLog("Unit: " + lpUnit);
        BrainVoyagerQX.PrintToLog("Value: " + lpValue);
    }

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyagerQX.PrintToLog("Open " + projfilenames[filecounter]);
            
            // high pass temporal filtering
            if (!noTHP) {
                // hack
                var name = new String(projfilenames[filecounter]);
                bvqxproject = BrainVoyagerQX.OpenDocument(name.toString());
                if (allTHP) {
                    success = bvqxproject.TemporalHighPassFilter(thpValue, thpUnit); 
                } else if (linearTHP) {
                    success = bvqxproject.LinearTrendRemoval();
                }
                if (success) {
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyagerQX.PrintToLog("Temporal high-pass filtering of this project did not succeed.");
                }
                bvqxproject.Close();
            }
            // low pass temporal filtering
            if (doTLP) {
                bvqxproject = BrainVoyagerQX.OpenDocument(projfilenames[filecounter]);
                success = bvqxproject.TemporalGaussianSmoothing(lpValue, lpUnit);
                if (success) {
                    //resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
                    resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                    BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
                    projfilenames[filecounter] = resultfilename; // place new name back in old name array
                } else {
                    BrainVoyagerQX.PrintToLog("Temporal low-pass filtering of this project did not succeed.");
                }
                bvqxproject.Close();
            }
        }
        BrainVoyagerQX.PrintToLog("Finished temporal filtering.");
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "temporalfiltering"); 
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}
 
}

scriptObj.startSpatialFiltering = function() { 

    var success = true;
    var sgsValue = sgsSizeBox.value;
    var sgsUnit;
    if (sgsMMbox.checked) {
        sgsUnit = "mm";
    } else {
        sgsUnit = "px";
    }
    BrainVoyagerQX.PrintToLog("Selected options for spatial low-pass filtering (gaussian smoothing): ");
    BrainVoyagerQX.PrintToLog("Unit: " + sgsUnit);
    BrainVoyagerQX.PrintToLog("Value: " + sgsValue);

    if (!cancel) {
        var filecounter, bvqxproject, resultfilename;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            BrainVoyagerQX.PrintToLog("Open " + projfilenames[filecounter]);
            bvqxproject = BrainVoyagerQX.OpenDocument(projfilenames[filecounter]);
            success = bvqxproject.SpatialGaussianSmoothing(sgsValue, sgsUnit);
            if (success) {
     //           resultfilename = this.getPath(projfilenames[filecounter]) + bvqxproject.FileNameOfPreprocessdFMR;
                resultfilename = bvqxproject.FileNameOfPreprocessdFMR;
                BrainVoyagerQX.PrintToLog("New file name: " + resultfilename);
                projfilenames[filecounter] = resultfilename; // place new name back in old name array
            } else {
                BrainVoyagerQX.PrintToLog("Spatial filtering for this project did not succeed.");
            } 
            bvqxproject.Close();
        }
        BrainVoyagerQX.PrintToLog("Finished spatial filtering.");
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "spatialfiltering");
    } else {
		BrainVoyagerQX.PrintToLog("Procedure was cancelled.");
	}
}
	
// ~~~~~~~~~~~~~~~~~~~~~~~ Create FMR/DMRs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    
scriptObj.createProjects = function() {
    
	// properties of ProjectFactory class
    var currentprojecttype;

    // declare and initialize member properties
	var filetype; 
	var firstfilename;
	var nrofvols; 
	var nrofvolsskip; 
	var createamr; 
	var nrslices; 
	var stcprefix; 
	var swapbytes; 	
	var mosaicx; 
	var mosaicy; 
	var nrbytes; 
	var savingdir;
	var volsinimg; 
	var imgresx; 
	var imgresy; 
    
		BrainVoyagerQX.PrintToLog("Retrieve create project parameters...");
    
        // parameters for determination of project type
        var isdmr = isDMRbox.checked;
        var ismosaic = isMosaicBox.checked;
        var isfmr = isFMRbox.checked;
        var looping = sliceloopsBox.checked;
        if (isdmr && ismosaic) {
            this.currentprojecttype = PROJECTTYPE_DMR_MOSAIC;
        } else if (isdmr) {
            this.currentprojecttype = PROJECTTYPE_DMR;    
        } else if (isfmr && looping) {
            this.currentprojecttype = PROJECTTYPE_FMR_LOOPING;
        } else if (isfmr && ismosaic) {
            this.currentprojecttype = PROJECTTYPE_FMR_MOSAIC;  
        } else if (isfmr) {
            this.currentprojecttype = PROJECTTYPE_FMR;
        }
        BrainVoyagerQX.PrintToLog("Project type: " + this.getProjectTypeString(this.currentprojecttype));
    
        // project-specific parameters
        this.filetype = filetypeBox.currentText; 
        this.nrofvols = nrVolsBox.value;
        this.nrofvolsskip = skipNrVolsBox.value;
        this.createamr = createAMRbox.checked; 
        this.nrslices = nrSlicesBox.value; 
        this.stcprefix = stcPrefixBox.text; 
        this.swapbytes = bigendianBox.checked; 	
        this.mosaicx = xMosResBox.value; 
        this.mosaicy = yMosResBox.value; 
        this.nrbytes = bytesPerPixelBox.value; 
        this.volsinimg = (nrMosaicVols.currentIndex+1); 
        this.imgresx = imgXResBox.value; 
        this.imgresy = imgYResBox.value; 
        
        var project;
        var filecounter, bvqxproject, newname;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            BrainVoyagerQX.PrintToLog("Start creating project " + (filecounter+1) + " from:\n" + projfilenames[filecounter]);
            this.firstfilename = projfilenames[filecounter];
            this.savingdir = this.getPath(this.firstfilename);
            newname = this.firstfilename.substring(0, (this.firstfilename.length-3)) + (isfmr ? "fmr" : "dmr"); 
            BrainVoyagerQX.PrintToLog("New name: " + newname);
                                            
            var projparams = "Project parameters:\n";
            projparams += "\tFiletype: " + this.filetype + "\n";
            projparams += "\tFirst file name: " + this.firstfilename + "\n";
            projparams += "\tNumber of volumes: " + this.nrofvols + "\n";
	    	projparams += "\tNumber of skipped volumes: " + this.nrofvolsskip + "\n";
		    projparams += "\tCreate pseudo AMR: " + this.createamr + "\n";
		    projparams += "\tNumber of slices: " + this.nrslices + "\n";
		    projparams += "\tSTC prefix: " + this.stcprefix + "\n";
		    projparams += "\tSwap bytes: " + this.swapbytes + "\n";
		    projparams += "\tSize of (mosaic) image (x-axis): " + this.mosaicx + "\n";
            projparams += "\tSize of (mosaic) image (y-axis): " + this.mosaicy + "\n";
            projparams += "\tNumber of bytes: " + this.nrbytes + "\n";
            projparams += "\tSaving directory: " + this.savingdir + "\n";
            projparams += "\tNumber of volumes in mosaic file: " + this.volsinimg + "\n";
            projparams += "\tSize of individual image (x-axis): " + this.imgresx + "\n";
            projparams += "\tSize of individual image (y-axis): " + this.imgresy + "\n";
            BrainVoyagerQX.PrintToLog(projparams);

            switch (this.currentprojecttype) {
      
                case PROJECTTYPE_FMR:
                    project = BrainVoyagerQX.CreateProjectFMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices, 
                        this.stcprefix, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes, this.savingdir);
                    break;
                case PROJECTTYPE_FMR_MOSAIC:
                    project = BrainVoyagerQX.CreateProjectMosaicFMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices, 
                        this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;
                case PROJECTTYPE_FMR_LOOPING:
                    project = BrainVoyagerQX.CreateProjectFMRSlicesTimeLooping(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr,         
                    this.nrslices, this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;  
                case PROJECTTYPE_DMR:
                    project = BrainVoyagerQX.CreateProjectDMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices, 
                        this.stcprefix, this.swapbytes, this.imgresx, this.imgresy, this.nrbytes, this.savingdir);
                    break;    
                case PROJECTTYPE_DMR_MOSAIC:    
                    project = BrainVoyagerQX.CreateProjectMosaicDMR(this.filetype, this.firstfilename, this.nrofvols, this.nrofvolsskip, this.createamr, this.nrslices, 
                        this.stcprefix, this.swapbytes, this.mosaicx, this.mosaicy, this.nrbytes, this.savingdir, this.volsinimg, this.imgresx, this.imgresy);
                    break;
                default:
                    BrainVoyagerQX.PrintToLog("Projecttype unknown.");
            }          

            if (istCheckBox.checked == 1) {
                BrainVoyagerQX.PrintToLog("Current InterSliceTime value: " + project.InterSliceTime);
                project.InterSliceTime = istValueBox.value;
                BrainVoyagerQX.PrintToLog("New InterSliceTime value: " + project.InterSliceTime);
            } 
            if (trCheckBox.checked == 1) {
                BrainVoyagerQX.PrintToLog("Current TR value: " + project.TR);
                project.TR = trValueBox.value;
                project.TimeResolutionVerified = true;
                BrainVoyagerQX.PrintToLog("New TR value: " + project.TR);
            }
            if (xCheckBox.checked == 1) {
                BrainVoyagerQX.PrintToLog("Current pixel x-resolution: " + project.PixelSizeOfSliceDimX);
                project.PixelSizeOfSliceDimX = xValueBox.value;
                BrainVoyagerQX.PrintToLog("New pixel x-resolution: " + project.PixelSizeOfSliceDimX);
                project.VoxelResolutionVerified = true;
            }
            if (yCheckBox.checked == 1) {
                BrainVoyagerQX.PrintToLog("Current pixel y-resolution: " + project.PixelSizeOfSliceDimY);
                project.PixelSizeOfSliceDimY = yValueBox.value;
                BrainVoyagerQX.PrintToLog("New pixel x-resolution: " + project.PixelSizeOfSliceDimY);
                project.VoxelResolutionVerified = true;
            }
            if (zCheckBox.checked == 1) {
                BrainVoyagerQX.PrintToLog("Current slice thickness: " + project.SliceThickness);
                project.SliceThickness = zValueBox.value; 
                BrainVoyagerQX.PrintToLog("New slice thickness: " + project.SliceThickness);
                project.VoxelResolutionVerified = true;   
            }

            project.SaveAs(newname);
            projfilenames[filecounter] = newname;
            project.Close();
         
    } // end filecounter
    if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_PROJECTCREATION, "projectcreation");
} 

// ~~~~~~~~~~~~~~~~~~~~~~~ Create VTC/VDWs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.createVTCsOrVDWs = function() {


    // get parameters
    var vmr;
    var doCreateVTC = rbtnSelectVTC.checked;
    var dataType = (rbtnVTCVDWdtypeFloat.checked ? 2 : 1); // 1: int16, 2: float32
    var resolution;
    if (rbtnResolution1mm3.checked) {
        resolution = 1;
    } else if (rbtnResolution2mm3.checked) {
        resolution = 2;
    } else {
        resolution = 3;
    }
    var interpolation           = (cmbVTCVDWInterpolation.currentIndex - 1); // 0: nearest neighbour, 1: trilinear, 2: sinc 
    var threshold               = spbIntensityThreshold.value;
    var useExtendedBoundingBox  = chbExtendCerebellum.checked;
    var vmrname                 = lblVTCVDWVMRname.text;
    
    // check files
    if (projfilenames.length < 1) {
        success = false;
        BrainVoyagerQX.PrintToLog("No files selected.");
    } else if (vmrname.length < 1) {
        BrainVoyagerQX.PrintToLog("No VMR name selected. Preprocessing " + (doCreateVTC ? "VTCs" : "VDWs") + " not possible.");
        success = false;
    } else {
                    
        // start processing
        var project;
        var filecounter, bvqxproject, newname;
        
        BrainVoyagerQX.PrintToLog("VTC/VDW creation parameters:\n" + "interpolation: " + interpolation + "\nthreshold: " + threshold);
        BrainVoyagerQX.PrintToLog("Use extended bounding box: " + (useExtendedBoundingBox ? "yes" : "no")); 
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
            
            BrainVoyagerQX.PrintToLog("Start creating VTC/VDW " + (filecounter+1) + " from:\n" + projfilenames[filecounter][0]); 

            // get filenames
            var nameSrc     = projfilenames[filecounter][0];
            var nameIAfile  = projfilenames[filecounter][1]; 
            var nameFAfile  = projfilenames[filecounter][2];   
            vmr = BrainVoyagerQX.OpenDocument(vmrname);     // in target space, right?
            // preparation for creating VTC name
            this.firstfilename = projfilenames[filecounter][0];
            this.savingdir = this.getPath(this.firstfilename);
            
            if (rbtnSelectNativeSpace.checked) {

                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_NATIVE.vtc" : "_NATIVE.vdw"); 
                
                // create VTC/VDW in native space
                vmr.ExtendedTALSpaceForVTCCreation = false; // non applicable for native space
                if (doCreateVTC) {
                    success = vmr.CreateVTCInVMRSpace(nameSrc, nameIAfile, nameFAfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInVMRSpace(nameSrc, nameIAfile, nameFAfile, newname, dataType, resolution, interpolation, threshold);
                }
   //             if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_NORMALISATION, (doCreateVTC ? "create_native_vtc" : "create_native_vdw"));
            
            } else if (rbtnSelectACPCSpace.checked) {

                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_ACPC.vtc" : "_ACPC.vdw"); 
                
                // create VTC in AC-PC space
                var nameACPCfile  = projfilenames[filecounter][3];                 
                vmr.ExtendedTALSpaceForVTCCreation = false; // non applicable for AC-PC space
                if (doCreateVTC) {
                    success = vmr.CreateVTCInACPCSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInACPCSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, newname, dataType, resolution, interpolation, threshold);                
                }
   //             if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_NORMALISATION, (doCreateVTC ? "create_acpc_vtc" : "create_acpc_vdw"));                

            } else if (rbtnSelectTalSpace.checked) {
            
                // create VTC/VDW name
                newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ((doCreateVTC) ? "_TAL.vtc" : "_TAL.vdw"); 
                
                // create VTC in Talairach space
                var nameACPCfile  = projfilenames[filecounter][3];  
                var nameTALfile  = projfilenames[filecounter][4];                  
                vmr.ExtendedTALSpaceForVTCCreation = useExtendedBoundingBox; // this is true or false
                if (doCreateVTC) {
                    success = vmr.CreateVTCInTALSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, nameTALfile, newname, dataType, resolution, interpolation, threshold);
                } else {
                    success = vmr.CreateVDWInTALSpace(nameSrc, nameIAfile, nameFAfile, nameACPCfile, nameTALfile, newname, dataType, resolution, interpolation, threshold);
                }
     //           
            }
            vmr.Close();
            projfilenames[filecounter][0] = newname;
            if (success) {
                BrainVoyagerQX.PrintToLog("Created: " + newname);                
            } else {
                BrainVoyagerQX.PrintToLog("VDW creation did not succeed.");
            }
        } // end loop
        
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_VTCPREPROCESSING, (doCreateVTC ? "resulting_vtc" : "resulting_vdw"));
        
    } // end if success
    return success;
}


// ~~~~~~~~~~~~~~~~~~~~~~~ Preprocess VTCs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.startPreprocessingVTCs = function() {
  
    // get parameters
    var SGSunit; // ("mm" or "vx")
    if (rbtnVTCmm.checked) {
        SGSunit = "mm";
    } else {
        SGSunit = "vx";
    } 
    var SGSquantity = dspbFWHMSpatialSmooth.value; 
    var HPFunit; //  ("cycles" or "Hz")
    if (rbtnVTCHz.checked) {
        HPFunit = "Hz";
    } else {
        HPFunit = "cycles";
    }
    var HPFquantity = dspbVTCFWHMTHP.value; 
    var LPFunit; // ("d" or "dps" (data points) or "s" or "secs" (seconds))
    if (rbtnVTCSec.checked) {
        LPFunit = "secs";
    } else {
        LPFunit = "dps";
    }
    var LPFquantity = dspbVTCFWHMTGS.value;
      
    // start processing
    var currentvtc;
    var success;
    var filecounter;
    
    var vmrname = lblVTCVDWVMRname.text;
    if (vmrname.length < 1) {
        BrainVoyagerQX.PrintToLog("No VMR name selected. Preprocessing VTCs not possible.");
        success = false;
    } else if (projfilenames.length < 1) {
        BrainVoyagerQX.PrintToLog("No VTC names available. Preprocessing VTCs not possible.");
        success = false;
    } else {
        var vmr;
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            vmr = BrainVoyagerQX.OpenDocument(vmrname);
            if (chbCreateVTCVDWs.checked) { // in this case we have a 2-dimensional matrix
                currentvtc = projfilenames[filecounter][0];
            } else {
                currentvtc = projfilenames[filecounter];
            }
            vmr.LinkVTC(currentvtc);
            BrainVoyagerQX.PrintToLog("Current VTC file: " + vmr.FileNameOfCurrentVTC);  // show name of current VTC

            // spatial low-pass filter
            if (chbVTCSpatialSmoothing.checked) {
                vmr.SpatialGaussianSmoothing(SGSquantity, SGSunit);    // FWHM value and unit ("mm" or "vx")
                BrainVoyagerQX.PrintToLog("Name of spatially smoothed VTC file: " + vmr.FileNameOfCurrentVTC);
                if (chbCreateVTCVDWs.checked) { 
                    projfilenames[filecounter][0] = vmr.FileNameOfCurrentVTC; // 0: project file 1, 2, etc: remainders of IA, FA files etc
                } else {
                    projfilenames[filecounter] = vmr.FileNameOfCurrentVTC;
                }
            }

            if (chbVTCTemporalFiltering.checked) {
                
                // temporal high-pass filter
                if (rbtnVTCAllTHP.checked) { // high-pass temporal filter (see below) includes LTR
                    vmr.TemporalHighPassFilter(HPFquantity, HPFunit);    // HP alue and unit ("cycles" or "Hz")
                    BrainVoyagerQX.PrintToLog("Name of VTC file without linear trends: " + vmr.FileNameOfCurrentVTC); 
                    if (chbCreateVTCVDWs.checked) { 
                        projfilenames[filecounter][0] = vmr.FileNameOfCurrentVTC; // 0: project file 1, 2, etc: remainders of IA, FA files etc
                    } else {
                        projfilenames[filecounter] = vmr.FileNameOfCurrentVTC;
                    }  
                } else if (rbtnVTCLinearHP.checked) {
                    vmr.LinearTrendRemoval();    // FWHM value and unit ("mm" or "vx")
                    BrainVoyagerQX.PrintToLog("Name of VTC file without linear trends: " + vmr.FileNameOfCurrentVTC);
                    if (chbCreateVTCVDWs.checked) { 
                        projfilenames[filecounter][0] = vmr.FileNameOfCurrentVTC; // 0: project file 1, 2, etc: remainders of IA, FA files etc
                    } else {
                        projfilenames[filecounter] = vmr.FileNameOfCurrentVTC;
                    }
                }
                
                // temporal low-pass filter
                if (rbtnVTCTLP.checked) {
                    vmr.TemporalGaussianSmoothing(LPFquantity, LPFunit);    // FWHM value and unit ("d" or "dps" (data points) or "s" or "secs" (seconds))
                    BrainVoyagerQX.PrintToLog("Name of temporally smoothed VTC file: " + vmr.FileNameOfCurrentVTC);
                    if (chbCreateVTCVDWs.checked) { 
                        projfilenames[filecounter][0] = vmr.FileNameOfCurrentVTC; // 0: project file 1, 2, etc: remainders of IA, FA files etc
                    } else {
                        projfilenames[filecounter] = vmr.FileNameOfCurrentVTC;
                    }
                }
            }
            vmr.Close();
        } // end loop   
        
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_VTCPREPROCESSING, "preprocessedvtcs");
        
    } // end if success
}


// ~~~~~~~~~~~~~~~~~~~~~~~ Create MTCs ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// the resulting textfile of the preprocessed VTCs can be used, but add the surface files following the line of each VTC filename
scriptObj.createMTCs = function() {

    BrainVoyagerQX.PrintToLog("Create MTC(s)...");
    
    // get parameters
    var samplingdepth = dspbMTCFrom.value;
    var samplingouter = dspbMTCTo.value; 
    BrainVoyagerQX.PrintToLog("Sampling from: " + samplingdepth.toString() + " to: " + samplingouter.toString()); 
    
    // start processing
    var currentvtc, vmr, newname, currentsrf;
    var success;
    var filecounter;
    var vmrname = lblMTCVMRname.text;  
    
    if (vmrname.length < 1) {
        success = false;
        BrainVoyagerQX.PrintToLog("No VMR or SRF selected. Cannot create MTCs.");
        
    } else if (projfilenames.length < 1) {
        BrainVoyagerQX.PrintToLog("No VTC filenames available. Cannot create MTCs.");
    } else {
        
            this.firstfilename = projfilenames[0][0];      
            this.savingdir = this.getPath(this.firstfilename);            
            
        for (filecounter = 0; filecounter < projfilenames.length; filecounter++) {
        
            newname = this.firstfilename.substring(0, (this.firstfilename.length-4)) + ".mtc";           
            vmr = BrainVoyagerQX.OpenDocument(vmrname);
            currentvtc = projfilenames[filecounter][0];           
            vmr.LinkVTC(currentvtc);
            srfname = projfilenames[filecounter][1];                        
            vmr.LoadMesh(srfname);
            BrainVoyagerQX.PrintToLog("Creating MTC from " + currentvtc);
            vmr.CreateMTCFromVTC(samplingdepth, samplingouter, newname); 
            projfilenames[filecounter][0] = newname;
            vmr.Close();
        }
        
        if (FORBVQX22) this.writeToTextFile(projfilenames, TEXTFILE_FOR_SURFACEPROCESSING, "mtcfiles");
    }
    return success;
}

// ~~~~~~~~~~~~~~~~~~~~~~~ main ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

scriptObj.initDlg = function() { 

    this.batchDlg.windowTitle = "batch processing wizard"; 
    
    // ~ global widgets
    cmbSelectProcSpace = BrainVoyagerQX.FindChild("cmbSelectProcSpace");
    // native space
    createProjectsBox = BrainVoyagerQX.FindChild("chbCreateProjects");
    doSliceTimeCorrBox = BrainVoyagerQX.FindChild("chbSCS");
    doMotionCorrBox = BrainVoyagerQX.FindChild("chbMOCO"); 
    doSpatialFilterBox = BrainVoyagerQX.FindChild("chbSF");
    doTempFilterBox = BrainVoyagerQX.FindChild("chbTF"); 
    mocoOnlyBox = BrainVoyagerQX.FindChild("rbtnMOCO");
    mocoAndISABox  = BrainVoyagerQX.FindChild("rbtnMOCOISA");
    thpAllBox  = BrainVoyagerQX.FindChild("rbtnAllTHP"); 
    thpLinearBox = BrainVoyagerQX.FindChild("rbtnLinearHP"); 
    thpNoneBox = BrainVoyagerQX.FindChild("rbtnNoTHP");
    tlpNoBox = BrainVoyagerQX.FindChild("rbtnNoTLP");
    tlpYesBox = BrainVoyagerQX.FindChild("rbtnTLP");
    sfNoBox = BrainVoyagerQX.FindChild("rbtnSFno");
    sfYesBox = BrainVoyagerQX.FindChild("rbtnSFyes");
    // normalised space
    chbCreateVTCVDWs        = BrainVoyagerQX.FindChild("chbCreateVTCVDWs");
    chbVTCSpatialSmoothing  = BrainVoyagerQX.FindChild("chbVTCSpatialSmoothing");
    chbVTCTemporalFiltering = BrainVoyagerQX.FindChild("chbVTCTemporalFiltering");
    rbtnVTCNoTHP            = BrainVoyagerQX.FindChild("rbtnVTCNoTHP"); 
    rbtnVTCLinearHP         = BrainVoyagerQX.FindChild("rbtnVTCLinearHP");
    rbtnVTCAllTHP           = BrainVoyagerQX.FindChild("rbtnVTCAllTHP");
    rbtnVTCNoTLP            = BrainVoyagerQX.FindChild("rbtnVTCNoTLP"); 
    rbtnVTCTLP              = BrainVoyagerQX.FindChild("rbtnVTCTLP");
    // surface space
    chbCreateMTCs           = BrainVoyagerQX.FindChild("chbCreateMTCs");
    
    // ~ widgets on create projects tab
    createAMRbox = BrainVoyagerQX.FindChild("chbCreateAMR");
    filetypeBox = BrainVoyagerQX.FindChild("cmbFileType"); 
    bigendianBox = BrainVoyagerQX.FindChild("chbBigEndian"); 
  //  selectSrcFilesBox = BrainVoyagerQX.FindChild("btnSelectSrcFiles"); 
    nrSlicesBox = BrainVoyagerQX.FindChild("spbNrSlices");
    nrVolsBox = BrainVoyagerQX.FindChild("spbNrOfVols"); 
    skipNrVolsBox = BrainVoyagerQX.FindChild("spbSkipNrVols"); 
    xMosResBox = BrainVoyagerQX.FindChild("spbXRes"); 
    yMosResBox = BrainVoyagerQX.FindChild("spbYRes"); 
    isMosaicBox = BrainVoyagerQX.FindChild("chbMosaic"); 
    nrMosaicVols = BrainVoyagerQX.FindChild("cmbNrMosVols"); 
    imgXResBox = BrainVoyagerQX.FindChild("spbImgXRes"); 
    imgYResBox = BrainVoyagerQX.FindChild("spbImgYRes"); 
    isDMRbox = BrainVoyagerQX.FindChild("rbtnDMR"); 
    isFMRbox = BrainVoyagerQX.FindChild("rbtnFMR"); 
    stcPrefixBox = BrainVoyagerQX.FindChild("leStcPrefix");
    sliceloopsBox = BrainVoyagerQX.FindChild("chbSlicesXTime"); 
 //   isFloatBox = BrainVoyagerQX.FindChild("chbStcFormatFloat"); 
  //  isIntBox = BrainVoyagerQX.FindChild("chbStcFormatInt"); 
    bytesPerPixelBox = BrainVoyagerQX.FindChild("spbBytesPerPixel");
    
    // ~ create projects: manual settings
    istCheckBox = BrainVoyagerQX.FindChild("chbIST"); 
    istValueBox = BrainVoyagerQX.FindChild("spbIST");
    trCheckBox = BrainVoyagerQX.FindChild("chbTR"); 
    trValueBox = BrainVoyagerQX.FindChild("spbTR"); 
    xCheckBox = BrainVoyagerQX.FindChild("chbXpix"); 
    xValueBox = BrainVoyagerQX.FindChild("dspbPixSizeX"); 
    yCheckBox = BrainVoyagerQX.FindChild("chbYpix"); 
    yValueBox = BrainVoyagerQX.FindChild("dspbPixSizeY"); 
    zCheckBox = BrainVoyagerQX.FindChild("chbZpix"); 
    zValueBox = BrainVoyagerQX.FindChild("dspbSliceThick");
    
    // ~ widgets on parameter tab
    mocoLessDataBox = BrainVoyagerQX.FindChild("chbLessData");
    mocoLogBox = BrainVoyagerQX.FindChild("chbLog"); 
    mocoInterpBox = BrainVoyagerQX.FindChild("cmbInterpMOCO"); 
    mocoNrIterBox = BrainVoyagerQX.FindChild("spbNumIter");
    mocoTargetVolBox = BrainVoyagerQX.FindChild("spbTargetVol");
    mocoISABox = BrainVoyagerQX.FindChild("rbtnMOCOISA");
    scsInterpBox = BrainVoyagerQX.FindChild("cmBoxInterpSCS"); 
    scsAscBox = BrainVoyagerQX.FindChild("rbtnAsc"); 
    scsAscIntBox = BrainVoyagerQX.FindChild("rbtnAscInt"); 
    scsAscInt2Box = BrainVoyagerQX.FindChild("rbtnAscInt2"); 
    scsDescBox = BrainVoyagerQX.FindChild("rbtnDesc"); 
    scsDescIntbox = BrainVoyagerQX.FindChild("rbtnDescInt"); 
    scsDescInt2Box = BrainVoyagerQX.FindChild("rbtnDescInt2");
    sgsMMbox = BrainVoyagerQX.FindChild("rbtnMm");
    sgsPxBox = BrainVoyagerQX.FindChild("rbtnPx"); 
    sgsSizeBox = BrainVoyagerQX.FindChild("dspbFWHMsizeSGS"); 
    thpCycBox = BrainVoyagerQX.FindChild("rbtnCyc"); 
    thpHzBox = BrainVoyagerQX.FindChild("rbtnHz"); 
    thpSizeBox = BrainVoyagerQX.FindChild("dspbFWHMTHP");
    lpSecBox = BrainVoyagerQX.FindChild("rbtnSec"); 
    lpTRBox = BrainVoyagerQX.FindChild("rbtnTR"); 
    lpSizeBox = BrainVoyagerQX.FindChild("dspbFWHMTHP");    
    
    // ~ widgets on Create VTC/VDW tab
    rbtnSelectVTC           = BrainVoyagerQX.FindChild("rbtnSelectVTC");
    rbtnSelectVDW           = BrainVoyagerQX.FindChild("rbtnSelectVDW");
    rbtnVTCVDWdtypeFloat    = BrainVoyagerQX.FindChild("rbtnVTCVDWdtypeFloat");
    rbtnVTCVDWdtypeInt      = BrainVoyagerQX.FindChild("rbtnVTCVDWdtypeInt");
    rbtnSelectNativeSpace   = BrainVoyagerQX.FindChild("rbtnSelectNativeSpace");
    rbtnSelectACPCSpace     = BrainVoyagerQX.FindChild("rbtnSelectACPCSpace");
    rbtnSelectTalSpace      = BrainVoyagerQX.FindChild("rbtnSelectTalSpace");
    chbExtendCerebellum     = BrainVoyagerQX.FindChild("chbExtendCerebellum");
    rbtnResolution1mm3      = BrainVoyagerQX.FindChild("rbtnResolution1mm3");
    rbtnResolution2mm3      = BrainVoyagerQX.FindChild("rbtnResolution2mm3");
    rbtnResolution3mm3      = BrainVoyagerQX.FindChild("rbtnResolution3mm3");
    cmbVTCVDWInterpolation  = BrainVoyagerQX.FindChild("cmbVTCVDWInterpolation");
    spbIntensityThreshold   = BrainVoyagerQX.FindChild("spbIntensityThreshold");
    
    // ~ widgets on Preprocess VTCs tab
    rbtnVTCmm               = BrainVoyagerQX.FindChild("rbtnVTCmm");
    rbtnVTCvox              = BrainVoyagerQX.FindChild("rbtnVTCvox"); 
    dspbFWHMSpatialSmooth   = BrainVoyagerQX.FindChild("dspbFWHMSpatialSmooth"); 
    rbtnVTCHz               = BrainVoyagerQX.FindChild("rbtnVTCHz"); 
    rbtnVTCCyc              = BrainVoyagerQX.FindChild("rbtnVTCCyc"); 
    dspbVTCFWHMTHP          = BrainVoyagerQX.FindChild("dspbVTCFWHMTHP"); 
    rbtnVTCSec              = BrainVoyagerQX.FindChild("rbtnVTCSec"); 
    rbtnVTCDataPoints       = BrainVoyagerQX.FindChild("rbtnVTCDataPoints"); 
    dspbVTCFWHMTGS          = BrainVoyagerQX.FindChild("dspbVTCFWHMTGS");
    
    // ~ widgets on Create MTCs tab
    dspbMTCFrom         = BrainVoyagerQX.FindChild("dspbMTCFrom");
    dspbMTCTo           = BrainVoyagerQX.FindChild("dspbMTCTo");
    
    // ~ widgets on files tab
    // native/scanner space
    getFileButton       = BrainVoyagerQX.FindChild("btnGetFiles");
    nroffilesBox        = BrainVoyagerQX.FindChild("spnBoxNrOfFiles");
    selectFilesNowBox   = BrainVoyagerQX.FindChild("rbtnFileDialog");
    selectTextFileBox   = BrainVoyagerQX.FindChild("rbtnTextFile");
    // normalised space
    selectFilesNowBox2   = BrainVoyagerQX.FindChild("rbtnFileDialog_3");
    selectTextFileBox2   = BrainVoyagerQX.FindChild("rbtnTextFile_3");
    btnGetVTCVDWnames   = BrainVoyagerQX.FindChild("btnGetVTCVDWnames");
    btnVTCVDWGetVMRName = BrainVoyagerQX.FindChild("btnVTCVDWGetVMRName");
    lblVTCVDWVMRname    = BrainVoyagerQX.FindChild("lblVTCVDWVMRname"); 
    spbNrVTCVDWs        = BrainVoyagerQX.FindChild("spbNrVTCVDWs");
    // surface space
    selectFilesNowBox3   = BrainVoyagerQX.FindChild("rbtnFileDialog_4");
    selectTextFileBox3   = BrainVoyagerQX.FindChild("rbtnTextFile_4");    
    btnGetMTCnames      = BrainVoyagerQX.FindChild("btnGetMTCnames"); 
    btnMTCGetVMRName    = BrainVoyagerQX.FindChild("btnMTCGetVMRName"); 
    lblMTCVMRname       = BrainVoyagerQX.FindChild("lblMTCVMRname"); 
    spbNrMTCs           = BrainVoyagerQX.FindChild("spbNrMTCs"); 

    // connect widgets to functions
	this.batchDlg.btnStart.clicked.connect(this, this.startProcessing); // might need to go to top of function
    getFileButton.clicked.connect(this, this.getNativeProcessingFiles);
    btnGetVTCVDWnames.clicked.connect(this, this.getNormalisedProcessingFiles);
    btnGetMTCnames.clicked.connect(this, this.getSurfaceProcessingFiles);
    btnVTCVDWGetVMRName.clicked.connect(this, this.getVTCVDWVMRName);    
    btnMTCGetVMRName.clicked.connect(this, this.getMTCVMRName);
    
    
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
